local BarCooldowns = CreateFrame("Frame")
local AceGUI = LibStub("AceGUI-3.0")
if not _G["BarCooldowns"] then
	_G["BarCooldowns"] = BarCooldowns
end
BarCooldowns.bars = {}
local function CopyTable(t)
	local a = {}
	for key, val in pairs(t) do
		a[key] = type(val) == "table" and CopyTable(val) or val
	end
	setmetatable(a, getmetatable(t))
	return a
end
local function chainprint(t, ...)
	local a={...}
	local pkey
	for index,key in ipairs(a) do
		print(pkey or "root".."->"..tostring(t))
		if t[key] then
			t = t[key]
			pkey = key
		else
			print(key.." is nil")
			break
		end
	end
end
local function round(num, places)
	return tonumber(string.format("%."..(places or 0).."f",num))
end
local findcache
local function tfind(t, skey, string, nod)
	if not nod then findcache = {} end
	if not string then string = "" end
	for key, data in pairs(t) do
		if key == skey then
			print(string.."."..key, data)
		end
		if type(data) == "table" and not findcache[tostring(data)] then
			findcache[tostring(data)] = true
			tfind(data, skey, string.."."..key, true)
		end
	end
end
local defaults = {
	spells={}
}
if not CD_DB then CD_DB = {profiles = {},data = {}} end
setmetatable(CD_DB.profiles, {__index=function(t,v) t[v] = v return t[v] end})
setmetatable(CD_DB.data, {__index=function(t,v) t[v] = CopyTable(defaults) return t[v] end})
BarCooldowns.configs = setmetatable({}, {__index=function(t,v)
	if not BarCooldowns.frame then return end
	local f = BarCooldowns.frame
	local group = AceGUI:Create("InlineGroup")
	f:AddChild(group)
	group:SetFullWidth(true)
	f:DoLayout()
	group:SetAutoAdjustHeight(true)
	group:SetLayout("Flow")
	local height = AceGUI:Create("EditBox")
	group:AddChild(height)
	height:SetLabel("Height")
	height:SetRelativeWidth(0.49)
	group.h = height
	local width = AceGUI:Create("EditBox")
	group:AddChild(width)
	width:SetLabel("Width")
	width:SetRelativeWidth(0.49)
	group.w = width
	local lock = AceGUI:Create("CheckBox")
	group:AddChild(lock)
	--chainprint(CD_DB, "data", profile, "spells", spell, "lock")
	lock:SetLabel("Lock movement")
	lock:SetDescription("Disable dragging for this spell bar")
	lock:SetRelativeWidth(0.49)
	group.l = lock
	local showoff = AceGUI:Create("CheckBox")
	group:AddChild(showoff)
	showoff:SetLabel("Show while off cooldown")
	showoff:SetDescription("Shows the empty bar while the spell is not on cooldown")
	showoff:SetRelativeWidth(0.49)
	group.s = showoff
	local remove = AceGUI:Create("Button")
	remove:SetFullWidth(true)
	group:AddChild(remove)
	remove:SetText("Remove")
	group.r = remove
	group.frame:Hide()
	t[v] = group
	return t[v]
end})
local function ReleaseSelf(widget) AceGUI:Release(widget) end
local profile
BarCooldowns:SetScript("OnUpdate", function() if UnitName("player") and GetRealmName() then profile = CD_DB.profiles[UnitName("player").." - "..GetRealmName()] BarCooldowns:SetScript("OnUpdate", nil) for spell in pairs(CD_DB.data[profile].spells) do Cooldowns:CreateBar(spell) end end end)

function BarCooldowns:UpdateVisibility()
	for key, frame in pairs(self.bars) do
		frame:UpdateVisibility()
	end
end
BarCooldowns:SetScript("OnEvent", function(self) self:UpdateVisibility() end)
BarCooldowns:RegisterEvent("SPELL_UPDATE_COOLDOWN")

function BarCooldowns:CreateBar(spell)
	BarCooldowns.bars[spell] = CreateFrame("Frame", nil, UIParent)
	local data = CD_DB.data[profile].spells[spell]
	local bar = BarCooldowns.bars[spell]
	bar:SetPoint(data.point, nil, data.relative, x, y)
	bar:SetHeight(data.height)
	bar:SetWidth(data.width)
	bar:SetMovable(true)
	bar:EnableMouse(true)
	bar:SetScript("OnMouseDown", function(self) if not data.lock then self:StartMoving() end end)
	bar:SetScript("OnMouseUp", function(self) if not data.lock then self:StopMovingOrSizing() data.point, _, data.relative, data.x, data.y = self:GetPoint() end end)
	function bar:UpdateVisibility()
		if GetSpellCooldown(spell) == 0 then self.showtex:Hide() if not data.showoff then self:Hide() else self:Show() end else self.showtex:Show() self:Show() end
	end
	bar.bgtex = bar:CreateTexture(nil, "BACKGROUND")
	bar.bgtex:SetAllPoints()
	bar.bgtex:SetTexture(0.6,0.6,1,0.3)
	bar.showtex = bar:CreateTexture(nil, "BACKGROUND")
	bar.showtex:SetPoint("TOPLEFT")
	bar.showtex:SetPoint("BOTTOMLEFT")
	bar.showtex:SetTexture(0.3,0.3,1,1)
	bar.text = bar:CreateFontString(nil, "ARTWORK")
	bar.text:SetFont("Fonts\\FRIZQT__.TTF", 12, "OUTLINE")
	bar.text:SetTextColor(0.8,0.8,0.8)
	bar.text:SetText(spell)
	bar.text:SetPoint("TOPLEFT", 5, 0)
	bar.text:SetPoint("BOTTOMLEFT", 5, 0)
	bar.text:SetJustifyH("LEFT")
	bar.cdtext = bar:CreateFontString(nil, "ARTWORK")
	bar.cdtext:SetFont("Fonts\\FRIZQT__.TTF", 12, "OUTLINE")
	bar.cdtext:SetTextColor(0.8,0.8,0.8)
	bar.cdtext:SetPoint("TOPRIGHT", -5, 0)
	bar.cdtext:SetPoint("BOTTOMRIGHT", -5, 0)
	bar.cdtext:SetJustifyH("RIGHT")
	bar:SetScript("OnUpdate", function(self) local start, duration = GetSpellCooldown(spell) if start == 0 then bar.showtex:Hide() bar.cdtext:SetText("READY") else local tr=start+duration-GetTime() bar.showtex:SetWidth(data.width*(tr/duration)) bar.cdtext:SetText(round(tr, 1).."s") end end)
	if (not data.showoff) and GetSpellCooldown(spell) == 0 then bar:Hide() end
end

function BarCooldowns:CreateForSpell(input)
	if tonumber(input) and GetSpellInfo(input) then input = GetSpellInfo(tonumber(input)) end
	if not GetSpellInfo(input) then print(input.." is not a valid spell") end
	if CD_DB.data[profile].spells[input] then
		print("A bar for "..input.." already exists")
	end
	CD_DB.data[profile].spells[input] = {lock=false, x=0, y=0, point="CENTER", relative="CENTER", width=180, height=15, showoff = false}
	self:CreateBar(input)
	self:UpdateConfig()
end

function BarCooldowns:UpdateConfig()
	local c = 1
	for n, frame in pairs(BarCooldowns.configs) do
		frame.frame:Hide()
	end
	for spell in pairs(CD_DB.data[profile].spells) do
		local group = self.configs[c]
		group:SetTitle(spell)
		group.h:SetText(tostring(CD_DB.data[profile].spells[spell].height))
		group.h:SetCallback("OnEnterPressed", function(widget, callback, text) widget:SetDisabled(true) widget:SetDisabled(false) CD_DB.data[profile].spells[spell].height = tonumber(text) end)
		group.w:SetText(tostring(CD_DB.data[profile].spells[spell].width))
		group.w:SetCallback("OnEnterPressed", function(widget, callback, text) widget:SetDisabled(true) widget:SetDisabled(false) CD_DB.data[profile].spells[spell].width = tonumber(text) end)
		group.l:SetValue(CD_DB.data[profile].spells[spell].lock)
		group.l:SetCallback("OnValueChanged", function(widget, callback, value) CD_DB.data[profile].spells[spell].lock = value end)
		group.s:SetValue(CD_DB.data[profile].spells[spell].showoff)
		group.s:SetCallback("OnValueChanged", function(widget, callback, value) CD_DB.data[profile].spells[spell].showoff = value BarCooldowns:UpdateVisibility() end)
		group.r:SetCallback("OnClick", function() CD_DB.data[profile].spells[spell] = nil BarCooldowns.bars[spell]:Hide() BarCooldowns.bars[spell] = nil BarCooldowns:UpdateConfig() end)
		group.frame:Show()
		c=c+1
	end
end

function BarCooldowns:OpenConfigGUI()
	if not self.frame then
		self.frame = AceGUI:Create("Frame")
		local frame = self.frame
		frame:SetTitle("BarCooldowns beta-1.0")
		frame:SetStatusText("BarCooldowns beta-1.0 configuration GUI")
		frame:SetLayout("List")
		frame:SetCallback("OnClose", function(widget) widget:Hide() end)
		local createbargroup = AceGUI:Create("SimpleGroup")
		frame:AddChild(createbargroup)
		createbargroup:SetLayout("Flow")
		createbargroup:SetFullWidth(true)
		local eb = AceGUI:Create("EditBox")
		createbargroup:AddChild(eb)
		eb:SetLabel("Spell:")
		eb:SetRelativeWidth(0.5)
		eb:SetFullHeight(true)
		local b = AceGUI:Create("Button")
		createbargroup:AddChild(b)
		b:SetText("Create")
		b:SetRelativeWidth(0.49)
		b:SetFullHeight(true)
		b.text = ""
		b:SetCallback("OnClick", function(widget) if not widget.text then return end BarCooldowns:CreateForSpell(widget.text) end)
		eb:SetCallback("OnEnterPressed", function(widget, callback, text) widget:SetDisabled(true) widget:SetDisabled(false) b.text = text end)
		self:UpdateConfig()
	else
		self.frame:Show()
	end
end

SlashCmdList["COOLDOWNS_GUI"] = function() BarCooldowns:OpenConfigGUI() end
SLASH_COOLDOWNS_GUI1 = "/cooldowns"